import os

import axipy
from PySide2.QtCore import QFile
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QDialog
from axipy import data_manager, QueryTable, Table, SelectionTable, LinearUnits, provider_manager, GeometryClass, Layer

from TopoSimplifier.ui.tools.add_helper.distance_helper import Distance
from TopoSimplifier.ui.tools.simplifier_tools import Simplifier


class TopoSimplifier(QDialog):
    def __init__(self,parent=None):
        super().__init__(parent)
        self.__ui=self.load_ui("main_dlg.ui",parent)
        self.__initUnits()
        self.__ui.cmb_tables.currentIndexChanged.connect(self.__change_table)
        self.__ui.com_units.currentIndexChanged.connect(self.__change_units)
        self.__tollerance = Distance(self.__ui.spin_box_tollerance.value(), self.__getSelectionUnit())
        self.__ui.spin_box_tollerance.valueChanged.connect(self.__change_value)
        self.__initListTables()
        self.__ui.tb_select_folder.clicked.connect(self.__select_path)
        self.__ui.pb_run.clicked.connect(self.__run)
        self.__ui.pb_cancel.clicked.connect(self.__cancel)
        self.__run_task=None
        self.__ui.finished.connect(self.__cancel)

    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def show(self):
        self.__ui.show()
    def exec_(self):
        self.__ui.exec_()
    def __initListTables(self):
        if data_manager.selection is not None and data_manager.selection.is_spatial:
            if GeometryClass.Polygons in data_manager.selection.class_geometries():
                name_sel=data_manager.selection.name
                self.__ui.cmb_tables.addItem(name_sel)
        data_objects=data_manager.objects
        if len(data_objects)==0:
            return
        for item in data_objects:
            if isinstance(item,SelectionTable) and item.is_spatial:
                name=item.name
                if name=="Выборка":
                    continue
                if GeometryClass.Polygons in item.class_geometries():
                    self.__ui.cmb_tables.addItem(item.name)
        for item in data_objects:
            if isinstance(item,Table) and not isinstance(item,SelectionTable) and item.is_spatial:
                if GeometryClass.Polygons in item.class_geometries():
                    self.__ui.cmb_tables.addItem(item.name)
        self.__ui.cmb_tables.setCurrentIndex(0)
    def __initUnits(self):
        unit_obj=axipy.LinearUnits.items()
        for unit in unit_obj:
            name=unit[0]
            if name=='degree':
                continue
            obj=unit[1]
            name_for_view=obj.localized_name
            self.__ui.com_units.addItem(name_for_view)
        unit_m=axipy.LinearUnits.get("m")
        self.__ui.com_units.setCurrentText(unit_m.localized_name)
    def __change_table(self,index):
        name_sel_tab=self.__ui.cmb_tables.currentText()
        tab_sel=data_manager.find(name_sel_tab)
        unit=tab_sel.coordsystem.unit
        if unit=='degree':
            self.__ui.com_units.setCurrentText(axipy.LinearUnits.m.localized_name)
        else:
            self.__ui.com_units.setCurrentText(unit.localized_name)
    def __change_units(self):
        select_units=self.__getSelectionUnit()
        self.__tollerance=self.__tollerance.to_unit(select_units)
        self.__ui.spin_box_tollerance.setValue(self.__tollerance.value)

    def __change_value(self,value):
        self.__tollerance.value=value
    def __select_path(self):
        path_out=axipy.save_file_dialog('MapInfo Tab (*.tab)',title='Выбрать tab для сохраения результата')
        if path_out is not None:
            self.__path_out=path_out
            self.__ui.pb_run.setEnabled(True)
            self.__ui.ln_path_out.setText(str(self.__path_out))
            self.__ui.ln_path_out.setCursorPosition(0)
    def __getSelectionUnit(self):
        unit_obj = axipy.LinearUnits.items()
        unit_loc_name=self.__ui.com_units.currentText()
        for item_unit in unit_obj:
            if item_unit[1].localized_name==unit_loc_name:
                return item_unit[1]
        return LinearUnits.m
    def __update_progress(self,value):
        self.__ui.progressBar.setValue(value)
    def __updateMessageProgress(self,message):
        self.__ui.progressBar.setFormat(message + " .. %p%")
    def __run(self):
        if self.__path_out.exists():
            try:
                provider_manager.tab.remove_table_files(str(self.__path_out))
            except:
                axipy.Notifications.push("Упрощение полигонов","Файл:"+str(self.__path_out)+" существует и его не возможно удалить")
                return
        self.__ui.pb_run.setEnabled(False)
        self.__ui.pb_cancel.setText("Прервать")
        self.__cls_runner=Simplifier(self.__ui.cmb_tables.currentText(),str(self.__path_out),self.__tollerance)
        self.__cls_runner.change_progress.connect(self.__update_progress)
        self.__cls_runner.change_message.connect(self.__updateMessageProgress)
        def run_process(t: axipy.Task):
            self.__cls_runner.run()
        self.__run_task=axipy.Task(run_process)
        self.__run_task.finished.connect(self.__finish_task)
        self.__run_task.start()
    def __cancel(self):
        if self.__run_task is None:
            self.__ui.close()
        else:
            self.__cls_runner.setCancel()

    def __finish_task(self):
        self.__ui.pb_run.setEnabled(True)
        self.__ui.pb_cancel.setText("Закрыть")
        self.__run_task=None
        if self.__cls_runner.isOk:
            tab=provider_manager.openfile(str(self.__path_out))
            axipy.mainwindow.add_layer_interactive(Layer.create(tab))
        self.__cls_runner=None






