import os
from pathlib import Path

from axipy import Feature, Geometry, Point, Line, LineString, GeometryCollection, Polygon
from osgeo import ogr


class TableCache:
    def __init__(self,tab,size=100):
        self.__tab=tab
        self.__cache_features=[]
        self.__size=size
    def add(self,ft):
        if isinstance(ft,list):
            if len(ft)==0:
                return
            for item_ft in ft:
                self.add(item_ft)
        else:
            if isinstance(ft,dict):
                self.__dictToFeature(ft,)
            else:
                self.__cache_features.append(ft)
        self.end()

    def __dictToFeature(self,obj):
        geo = obj['geometry']
        if geo is None:
            return
        obj.pop('geometry')
        ft = Feature(obj, geometry=geo)
        self.__cache_features.append(ft)
    def end(self,all_write=False):
        if all_write and len(self.__cache_features)>0:
            self.__tab.insert(self.__cache_features)
            self.__cache_features.clear()
            return
        if len(self.__cache_features)>self.__size:
            self.__tab.insert(self.__cache_features)
            self.__cache_features.clear()
def getAttFeature(items_ft):
    att={}
    for item in items_ft:
        name=item[0]
        value=item[1]
        if name=='+geometry' or name=='+style':
            continue
        att[name]=value
    return att
def CountPoints(geo:Geometry)->int:
    count=0
    if isinstance(geo,Point):
        return 1
    if isinstance(geo,Line):
        return 2
    if isinstance(geo,LineString):
        count=len(geo.points)
        return count
    if isinstance(geo,Polygon):
        count=len(geo.points)
        for hole in geo.holes:
            count=count+len(hole)
        return count
    if isinstance(geo,GeometryCollection):
        count=0
        for item in geo:
            count=count+CountPoints(item)
        return count
    return 0
