import axipy
from PySide2.QtCore import QObject, QTemporaryDir, Signal
from axipy import data_manager, Schema, Attribute, provider_manager, Feature, Geometry
from loguru import logger
from osgeo import ogr


from TopoSimplifier.ui.tools.add_helper.distance_helper import Distance, getToleranceFromGrad
from TopoSimplifier.ui.tools.add_helper.helper_simplifier import getEdge, getEdgeExternal, createObject
from TopoSimplifier.ui.tools.add_helper.helper_table import TableCache, getAttFeature, CountPoints


class CalcIntProcent:
    def __init__(self,count):
        self.__last_proc=-1
        self.__count=count
    def getValue(self,curent):
        cur_val=int((curent/self.__count)*100+0.5)
        if cur_val==self.__last_proc:
            return None
        self.__last_proc=cur_val
        return cur_val

class Simplifier(QObject):
    change_progress=Signal(int)
    change_message=Signal(str)
    __row_id_field='__rowid__'
    def __init__(self,name_tab_source,path_result_tab,tollerance:Distance,folder_temp=None):

        super().__init__()
        self.__name_tab_source=name_tab_source
        self.__path_out=path_result_tab
        self.__tollerance=tollerance
        self.__isCancel=False
        self.isOk=False
    def setCancel(self):
        self.__isCancel=True
    def run(self):
        try:
            self.__tempFolder=QTemporaryDir()
            name_topo_tab=self.__name_tab_source+"__topo.tab"
            path_topo_tab=self.__tempFolder.filePath(name_topo_tab)
            sql="Select RowNum() as "+self.__row_id_field+",* from "+self.__name_tab_source
            query_tab=data_manager.query_hidden(sql)

            schema = Schema(
                Attribute.integer("id_base"),
                Attribute.integer('id'),
                coordsystem=query_tab.coordsystem
            )
            self.change_message.emit("Выборка исходных данных")
            tab_out = provider_manager.createfile(path_topo_tab, schema)
            geo_combine = None
            tab_out_cache=TableCache(tab_out)
            count=query_tab.count()
            count_source=count
            cls_procent=CalcIntProcent(count)
            self.change_message.emit("Поиск общих сегментов")


            def fun_progress(index):
                value = cls_procent.getValue(index)
                if value is not None:
                    self.change_progress.emit(value)
                if self.__isCancel:
                    return True
                return False
            for i,ft in enumerate(query_tab.items()):
                value_proc=cls_procent.getValue(i+1)
                if value_proc is not None:
                    self.change_progress.emit(value_proc)
                if self.__isCancel:
                    query_tab.close()
                    tab_out.commit()
                    tab_out.close()
                    return
                id_base = ft[self.__row_id_field]
                if ft.geometry is None:
                    continue
                if geo_combine is None:
                    geo_combine = ft.geometry
                else:
                    geo_combine = geo_combine.union(ft.geometry)
                result = getEdge(query_tab, id_base,self.__row_id_field, ft.geometry)
                if result is None:
                    continue
                tab_out_cache.add(result)
            ''' Получаем внешние face'''
            geo_combine_linestring=geo_combine.to_linestring()
            if geo_combine_linestring is not None:
                self.change_message.emit("Поиск внешних сегментов")
                self.change_progress.emit(0)
                external_edge = getEdgeExternal(query_tab, 0, self.__row_id_field,geo_combine_linestring,fun_progress)
            else:
                external_edge=None
                ft_comb=Feature({'id_base':0},geometry=geo_combine)
                tab_out_cache.add(ft_comb)
            if external_edge is not None:
                tab_out_cache.add(external_edge)
            tab_out_cache.end(True)
            tab_out.commit()
            if self.__isCancel:
                query_tab.close()

                tab_out.close()
                return
                #query_tab.close()
            self.change_progress.emit(0)
            work_coordsystem=tab_out.coordsystem
            if work_coordsystem.lat_lon:
                tollerance_coordsystem=getToleranceFromGrad(query_tab.get_bounds(),query_tab.coordsystem,self.__tollerance)
            else:
                tollerance_coordsystem=self.__tollerance.to_unit(work_coordsystem.unit)
            ''' Упрощаем geometry'''
            name_simplifier_tab = self.__name_tab_source + "__simplifier.tab"
            path_simplifier_tab = self.__tempFolder.filePath(name_simplifier_tab)
            tab_simlifier=provider_manager.createfile(path_simplifier_tab, tab_out.schema)
            tab_simply_cache = TableCache(tab_simlifier)
            self.change_message.emit("Упрощение сегментов")

            for i,ft in enumerate(tab_out.items()):
                value_proc = cls_procent.getValue(i + 1)
                if value_proc is not None:
                    self.change_progress.emit(value_proc)
                if self.__isCancel:
                    break
                geo = ft.geometry
                if geo is None:
                    continue
                geo_ogr = ogr.CreateGeometryFromWkb(geo.to_wkb())
                if geo_ogr is None:
                    continue
                if isinstance(tollerance_coordsystem,Distance):
                    geo_simplify = geo_ogr.SimplifyPreserveTopology(tollerance_coordsystem.value)
                else:
                    geo_simplify = geo_ogr.SimplifyPreserveTopology(tollerance_coordsystem)
                geo_axi_simlify = Geometry.from_wkb(geo_simplify.ExportToWkb(), coordsystem=tab_out.coordsystem)
                geo_ogr = None
                geo_simplify = None
                att = getAttFeature(ft.items())
                ft_new = Feature(att, geometry=geo_axi_simlify)
                tab_simply_cache.add(ft_new)
            tab_simply_cache.end(True)
            tab_simlifier.commit()
            tab_out.close()
            count_simplifyer=tab_simlifier.count()
            if count_source!=count_simplifyer:
                axipy.Notifications.push("Упрощение полигонов","Операция выполнена не корректно. Возможная причина - ошибки топологии")
            self.change_message.emit("Обновление объектов")
            source_pnt,result_pnt=self.finishedSimplifier(query_tab,tab_simlifier)
            axipy.Notifications.push("Упрощение полигонов","Точек до упрощениея:"+str(source_pnt)+" После упрощения:"+str(result_pnt))
            tab_simlifier.close()
            query_tab.close()
            self.isOk=True
        except Exception as e:
            logger.exception(e)
        return

    def finishedSimplifier(self,qyery_tab,simplifier_tab):
        source_pnt=0
        result_pnt=0
        tab_source=data_manager.find(self.__name_tab_source)
        tab_result=provider_manager.createfile(self.__path_out, tab_source.schema)
        tab_cache_result=TableCache(tab_result)
        cls_procent = CalcIntProcent(qyery_tab.count())
        for i,ft_source in enumerate(qyery_tab):
            value_proc = cls_procent.getValue(i + 1)
            if value_proc is not None:
                self.change_progress.emit(value_proc)
            id_rect=ft_source[self.__row_id_field]
            if id_rect==710:
                jkl=0
            dect_find = {"name": 'id_base', 'second': 'id', 'value': id_rect}
            geo_comb = createObject(dect_find, simplifier_tab,self.__row_id_field)
            if geo_comb is None:

                continue
            source_pnt=source_pnt+CountPoints(ft_source.geometry)
            result_pnt=result_pnt+CountPoints(geo_comb)
            att = getAttFeature(ft_source.items())
            att.pop(self.__row_id_field)
            ft_new=Feature(att,geometry=geo_comb)
            tab_cache_result.add(ft_new)
        tab_cache_result.end(True)
        tab_result.commit()
        tab_result.close()
        return source_pnt,result_pnt




